// Builtins command
// Sébastien Gendre <seb@k-7.ch>


#include "builtins.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>


/* Check if a command is builtin */
int builtin_is_command_builtin(int argc, char *argv[]) {
  if (strcmp(argv[0], "exit") == 0) {
    return 1;
  } else if (strcmp(argv[0], "cd") == 0) {
    return 1;
  } else if (strcmp(argv[0], "pwd") == 0) {
    return 1;
  } else if (strcmp(argv[0], "rls") == 0) {
    return 1;
  } else {
    return 0;
  }
}


/* Execute a builtin command */
int builtin_exec_builtin_command(int argc, char *argv[]) {
  if (strcmp(argv[0], "exit") == 0) {
    // Exec the builtin command exit
    builtin_exit(argc, argv);
  } else if (strcmp(argv[0], "cd") == 0) {
    // Exec the builtin command cd
    return builtin_cd(argc, argv);
  } else if (strcmp(argv[0], "pwd") == 0) {
    // Exec the builtin command pwd
    return builtin_pwd(argc, argv);
  } else if (strcmp(argv[0], "rls") == 0) {
    // Exec the builtin command 
    return builtin_rls(argc, argv);
  } else {
    // Return -1
    return -1;
  }
}


/* Exit bultin command */
void builtin_exit(int argc, char *argv[]) {
  if (argc == 2) {
    // If exit command is followed by a number, return it as exit code
    exit(atoi(argv[1]));
  }
  else {
    exit(EXIT_SUCCESS);
  }
}


/* cd bultin command */
int builtin_cd(int argc, char *argv[]) {
  char *new_wdir = NULL;
  struct passwd *pw_entry = NULL;
  int chdir_return = 0;

  if (argc == 1) {
    // No directory sepcified by the user, use the user home
    // Get user record in the password  database, from is uid
    pw_entry = getpwuid(getuid());
    // Get user home from the record
    new_wdir = pw_entry->pw_dir;
    // No free() on the returned pointer of getpwuid(), as say by the
    // manual of GETPWNAM(3)
  } else {
    // Get the new directory sepcified by the user
    new_wdir = argv[1];
  }

  // Set new working directory and return the value of this function
  chdir_return = chdir(new_wdir);
  if (chdir_return == 0) {
    return EXIT_SUCCESS;
  } else {
    perror("Error when set new working directory");
    return errno;
  }
}


/* pwd builtin command */
int builtin_pwd(int argc, char *argv[]) {
  char *wdir = get_current_dir_name();
  if (wdir != NULL) {
    printf("%s\n", get_current_dir_name());
    return EXIT_SUCCESS;
  } else {
    perror("Error while get working directory");
    return EXIT_FAILURE;
  }
}


/* rls builtin command */
int builtin_rls(int argc, char *argv[]) {
  printf("Not implemented yet");
  return EXIT_FAILURE;
}
