// Builtins command
// Sébastien Gendre <seb@k-7.ch>

#ifndef BUILTINS_H
#define BUILTINS_H

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <pwd.h>


/* Check if a command is builtin
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 *
 * Return:
 * 1 if the command is a builtin, else 0
*/
int builtin_is_command_builtin(int argc, char *argv[]);

/* Execute a builtin command
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 *
 * Return:
 * Return code of the bultin command, -1 if bultin command not found
*/
int builtin_exec_builtin_command(int argc, char *argv[]);

/* Exit bultin command
 *
 * This command accept one optionnal argument in argv: The code to exit to.
 * If the user exec `exit 2` command, the shell exit with the code `2`.
 * If the user only exec `exit` command, the shell exit with the code EXIT_SUCCESS
 * 
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command argumants
 *
 * Return:
 * Nothing, this exit the main process
*/
void builtin_exit(int argc, char *argv[]);


/* cd bultin command
 *
 * This command accept one optionnal argument in argv: The new path to working directory
 * If the user exec `cd /var` command, the shell set `/var` as the new working directory
 * If the user exec `cd`, the shell simply set the user home directory as the new working directory
 * 
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command argumants
 *
 * Return:
 * 0 if success, errno if error
*/
int builtin_cd(int argc, char *argv[]);


/* pwd builtin command
 *
 * This command simply print the working directory path
 * 
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command argumants
 *
 * Return:
 * 0 if success, 1 if error
*/
int builtin_pwd(int argc, char *argv[]);

/* rls builtin command
 *
 * This command list recurcively current direcotory and all its sub
 * directories
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command argumants
 *
 * Return:
 * 0 if success, 1 if error

*/
int builtin_rls(int argc, char *argv[]);



#endif /* BUILTINS_H */
