// All things related to interfaces (user input, user output, files, etc)
// Sébastien Gendre <seb@k-7.ch>

#include "interfaces.h"

//Wait for the user input and return it
//OUT:
//  user_input: a string with the user input. The user should take care of the
//              allocation.
//  return: 0 if an empty input was given, -1 if the function failed to read stdin
int interface_ask_user_input(char* user_input) {
  // Show the prompt and get user input
  printf(PROMPT_DEFAULT);
  printf(" ");
  if(fgets(user_input, LIMIT_USER_INPUT_MAX_LENGTH, stdin) == NULL)
    return -1;

  //Ensure that the end of line chars are removed
  user_input[strcspn(user_input, "\r\n")] = 0;

  return user_input[0];
}


//Print command for debugging
void interface_print_command(cmd_t* cmd) {
  printf("Command:\n");
  printf("\t- Name: %s\n", cmd->argv[0]);
  printf("\t- Arguments count: %d\n", cmd->argc - 1);
  if(cmd->argc == 1)
    printf("\t- No arguments\n");
  else {
    printf("\t- Arguments:\n");
    for (int i = 1; i < cmd->argc; ++i) {
      printf("\t\t- %s\n", cmd->argv[i]);
    }
  }
  printf("\t- Foreground: ");
  printf(cmd->foreground ? "true\n" : "false\n");
}
