// Jobs manipulation
// Sébastien Gendre <seb@k-7.ch>

#ifndef JOBS_H
#define JOBS_H

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

/* Start a given command, return its pid
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 * 
 * Return:
 * - The command pid, or -1 in case of fork error
*/
pid_t job_start_command(int argc, char *argv[]);

/* Wait the end of a given command targeted by its pid
 *
 * Parameters:
 * - pid: The command process id
 * 
 * Return:
 * - The command return code (0 is success)
 * - -1 In case the command called have not exited
*/
int job_wait_command_pid(pid_t pid);

/* Execute a given command as job
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 * 
 * Return:
 * - The command return code (0 is success)
 * - -1 In case the command called have not exited
*/
int job_exec_command(int argc, char *argv[]);


/* Search file redirection argument ('>') in argv
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 *
 * Return:
 * The position of '>' argument in the argv, -1 if not found
*/
int job_write_redirection_position_in_argv(int argc, char *argv[]);

/* Get file path where user ask to redirect stdout, if defined by the
 * user with a '>'
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 *
 * Return:
 * The file path where user ask to redirect stdout
 * If not defined, return NULL
*/
char* job_write_redirection_path_in_argv(int argc, char *argv[]);

/* Clean arguments after stdout redirection
 *
 * Parameters:
 * - argc: The command arguments counter
 * - argv: The command arguments 
 *
 * Return:
 * 0 if success, -1 if error
*/
int job_clean_argv_after_redirection_char(int *argc, char **argv);


#endif /* JOBS_H */
